classdef Beamformer
    % BEAMFORMER Delay-And-Sum Beamformer class
    %   Handles positions of microphones and time difference calculations
    
    properties
        soundVelocity = 340;
        samplerate;
        basePosition;
        receiverDistance;
        receiverPositions;
    end
    
    methods
        
        function bf = Beamformer(parBasePosition, ...
                parReceiverDistance, parSamplerate) % BEAMFORMER Construct an instance of this class
            
            bf.basePosition = parBasePosition;
            bf.receiverDistance = parReceiverDistance;
            if ~exist('parSamplerate','var') || isempty(parSamplerate)
                bf.samplerate = 16000; % default samplerate
            else
                bf.samplerate = parSamplerate;
            end
            
            r = parReceiverDistance/2;
            bf.receiverPositions = [ ...
                parBasePosition - [r 0 0]; ...
                parBasePosition + [r 0 0]; ...
                parBasePosition - [0 r 0]; ...
                parBasePosition + [0 r 0]];
        end
        
        function theta = getDirection(bf, signals)         % GETDIRECTION Direction of Arrival estimation
    
            
            % Get time difference of arrival
            tau1 = gccphat(signals(1,:)', signals(2,:)', bf.samplerate);
            tau2 = gccphat(signals(3,:)', signals(4,:)', bf.samplerate);
            
            % Reference time difference of arrival
            tau0 = bf.receiverDistance / bf.soundVelocity;

            % Calculate two direction proposals
            theta1 = asind(tau1/tau0);
            theta2 = asind(tau2/tau0);

            % Determine optimal guess (least error)
            if abs(theta1) < abs(theta2)
                if theta2 > 0
                    theta = mod(-(theta1 + 270), 360);
                else
                    theta = mod(theta1 + 270, 360);
                end
            else
                if theta1 > 0
                    theta = mod(theta2 + 360, 360);
                else
                    theta = mod(-theta2 + 180, 360);
                end
            end
        end
        
        function bfSignal = getBeamformedSignal(bf, signals, direction)
            % GETBEAMFORMEDSIGNAL Delay-And-Sum Beamforming for target
            % direction
            
            % Calculate delays
            r = bf.receiverDistance / (2 * bf.soundVelocity);


            % Delay-And-Sum Beamforming

        end
        
    end % methods
    
end % classdef
